/*
    Copyright (C) 2012  Leonardo Pozzati
    This file is part of MetastableRandWalk.

    MetastableRandWlak is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    MetastableRandWalk is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Nome-Programma.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MYLIBDENS_H
#define _MYLIBDENS_H


#include <list>
#include <map>
#include <utility>
#include <cmath>
#include <cstdlib>


//FRAME DENS
//funzione che determina la cornice che contiene le coordinate (x, y)
//dato il passo step in cui la griglia viene suddivisa
unsigned long frRecog(unsigned long step,
					  signed long x,
					  signed long y);

//funzione che conta il numero di siti interni (bordo compreso)
//ad un quadrato il cui lato dista side dal centro della griglia,
//che coincide con il centro del quadrato
unsigned long squareSites(unsigned long side);

//inizializzazione della mappa
//<numero di frame, <particelle, fattore di normalizzazione>
std::map<unsigned long, std::pair <unsigned long, unsigned long > >
frCountVsNormInit(unsigned long bound, unsigned long step);

void frCountVsNormUpdate(unsigned long step,
						 std::list< std::pair<signed long,
											  signed long> > state,
						 std::map<unsigned long,
								  std::pair<unsigned long,
											unsigned long> >
						 &frCountVsNorm);

void frCountVsNormReset(std::map<unsigned long,
								 std::pair<unsigned long,
										   unsigned long> >
						&frCountVsNorm);

//CROWN DENS
//funzione che determina la corona che contiene le coordinate (x, y),
//dato il limite superiore bound del lato della griglia
//e il passo step in cui questa viene divisa
unsigned long crRecog(unsigned long bound,
					  unsigned long step,
					  signed long x,
					  signed long y);

//verifica se un punto di coordinate (x,y)
//sia o meno contenuto nella circonferenza di raggio r
int externalSitesCheck(signed long x,
					   signed long y,
					   unsigned long r);

//funzione che conta il numero di siti interni (bordo compreso)
//alla circonferenza di raggio r (multiplo del passo)
//centrata nel centro della griglia
unsigned long circleSites(unsigned long r);

//inizializzazione della mappa
//<numero di corona, <particelle, fattore di normalizzazione>
std::map<unsigned long, std::pair <unsigned long, unsigned long > >
crCountVsNormInit(unsigned long bound, unsigned long step);

void crCountVsNormUpdate(unsigned long bound,
						 unsigned long step,
						 std::list< std::pair<signed long,
											  signed long> > state,
						 std::map<unsigned long,
								  std::pair<unsigned long,
											unsigned long> >
						 &crCountVsNorm);

void crCountVsNormReset(std::map<unsigned long,
								 std::pair<unsigned long,
										   unsigned long> >
						&crCountVsNorm);

#endif
