/*
    Copyright (C) 2012  Leonardo Pozzati
    This file is part of MetastableRandWalk.

    MetastableRandWlak is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    MetastableRandWalk is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Nome-Programma.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "libEvolSmart.h"

using namespace std;

							/* LIBEVOL */
list< pair <signed long, signed long> >
stateInit(unsigned long totalPart, unsigned long bound) {
	list< pair <signed long, signed long> > sysState;
	pair <signed long, signed long> coord;
	for(unsigned long particle=1; particle<=totalPart; particle++) {
		coord.first = (lrand48() % (2*bound + 1)) - bound;
		coord.second = (lrand48() % (2*bound + 1)) - bound;
		sysState.push_back(coord);
	}
	return sysState;
}

list< pair <signed long, signed long> >
noTrappedStateInit(unsigned long totalPart, unsigned long bound) {
	list< pair <signed long, signed long> > sysState;
	pair <signed long, signed long> coord;
	for(unsigned long particle=1; particle<=totalPart; ) {
		coord.first = (lrand48() % (2*bound + 1)) - bound;
		coord.second = (lrand48() % (2*bound + 1)) - bound;
		if ( coord.first || coord.second ) {
			sysState.push_back(coord);
			++particle;
		}
	}
	return sysState;
}

int originCheck(pair <signed long, signed long> coord,
				pair <signed long, signed long> positionToCheck) {
	return (coord == positionToCheck);
}

void stateResize(list< pair <signed long, signed long> > &state,
				 unsigned long &trappedCounter) {
	list< pair <signed long, signed long> >::iterator it1;
	pair < signed long, signed long> origin (0, 0);
	for(it1=state.begin(); it1!=state.end(); /*it1++*/) {
		if((*it1) == origin) {
			trappedCounter +=1;
			it1 = state.erase(it1);
		} else {
			it1++;
		}
	}
}

int viscTest(long double num) {
	return (drand48()>num);
}

void trapEmission(list< pair <signed long, signed long> > &state,
				  unsigned long &trappedCounter,
				  unsigned long &emissionCounter) {
	trappedCounter -= 1;
	emissionCounter += 1;
	int mossa;
	mossa = lrand48() % 4;
	pair <signed long, signed long> coordinates (0, 0);
	if(mossa == 0) {
		coordinates.first = 1;
	} else if(mossa == 1) {
		coordinates.second= 1;
	} else if(mossa == 2) {
		coordinates.first = -1;
	} else
		coordinates.second = -1;
	state.push_back(coordinates);
}

void makeSmartMove(signed long max,
				   unsigned long step,
				   list< pair <signed long, signed long> > &state,
				   map<unsigned long,
					   pair <unsigned long, unsigned long > >
				   &frCountVsNorm,
				   map<unsigned long,
					   pair <unsigned long, unsigned long > >
				   &crCountVsNorm,
				   unsigned long &trappedPartCounter) {
	list< pair <signed long, signed long> >::iterator it1;
	pair< signed long, signed long> origin (0, 0);
	int mossa;
	unsigned long frame, crown;
	for(it1=state.begin(); it1!=state.end(); /* it1++ */) {
		mossa = lrand48() % 4;
		//muovo la particella a seconda del valore di mossa
		if(mossa == 0) {
			(*it1).first += 1;
			(*it1).first = (*it1).first > max ? -max : (*it1).first;
		} else if(mossa == 1) {
			(*it1).second += 1;
			(*it1).second = (*it1).second > max ? -max : (*it1).second;
		} else if(mossa == 2) {
			(*it1).first -= 1;
			(*it1).first = (*it1).first < -max ? max : (*it1).first;
		} else {
			(*it1).second -= 1;
			(*it1).second = (*it1).second < -max ? max : (*it1).second;
		}
		if((*it1) == origin) {
			it1 = state.erase(it1);
			trappedPartCounter += 1;
		} else {
			frame = frRecog(step, (*it1).first, (*it1).second);
			crown = crRecog(max, step, (*it1).first, (*it1).second);
			(frCountVsNorm[frame]).first += 1;
			(crCountVsNorm[crown]).first += 1;
			it1++;
		}
	}
}

void evolveAndPrint(ofstream &frFile,
					ofstream &crFile,
					list< pair <signed long, signed long> > &state,
					unsigned long initialTrappedPart,
					unsigned long &emissionCounter,
					unsigned long &trappingEventCounter,
					unsigned long bound,
					unsigned long step,
					long double visc,
					unsigned long length1,
					unsigned long printTime1,
					unsigned seed) {

	unsigned long totalTrapped = initialTrappedPart;
	unsigned long partialTrapped = 0;

	//creazione delle mappe {cr,fr}CountVsNorm
	//<zona, <conteggio particelle, fattore di normalizzazione> >
	map<unsigned long, pair <unsigned long, unsigned long > >
	crCountVsNorm = crCountVsNormInit(bound, step);
	map<unsigned long, pair <unsigned long, unsigned long > >
	frCountVsNorm = frCountVsNormInit(bound, step);

	//Aggiornamento delle map {cr,fr}CountVsNorm
	crCountVsNormUpdate(bound, step, state, crCountVsNorm);
	frCountVsNormUpdate(step, state, frCountVsNorm);

	//stampa delle densità al tempo 0
	crDensPrint(crFile, crCountVsNorm, totalTrapped, 0);
	frDensPrint(frFile, frCountVsNorm, totalTrapped, 0);

	//reset dei conteggi delle particelle
	crCountVsNormReset(frCountVsNorm);
	frCountVsNormReset(crCountVsNorm);

	//reinizializzazione dei valori di normalizzazione,
	//in {cr,fr}CountVsNorm
	//per considerare la media temporale
	map<unsigned long, pair <unsigned long, unsigned long > >::iterator it1;
	for(it1=frCountVsNorm.begin(); it1!=frCountVsNorm.end(); it1++) {
		((*it1).second).second *= printTime1;
	}
	for(it1=crCountVsNorm.begin(); it1!=crCountVsNorm.end(); it1++) {
		((*it1).second).second *= printTime1;
	}
	
	//inizio dell'algoritmo di evoluzione
	unsigned long tempo;
	for(unsigned long ciclo=1; ciclo<=length1/printTime1; ciclo++) {
		tempo = (ciclo - 1) * printTime1 + 1;
		for(; tempo<=ciclo * printTime1; tempo++) {
			//aggiornamento delle coordinate per ogni particella
			
			makeSmartMove(bound, step, state,
						  frCountVsNorm, crCountVsNorm,
						  partialTrapped);
			//se nella trappola ci sono particelle
			//se si supera il test
			//una particella viene emessa
			if(totalTrapped != 0) {
				if(viscTest(visc)) {
					trapEmission(state, totalTrapped, emissionCounter);
				}
			}
			//aggiorno le totalTrap con le partialTrap
			totalTrapped += partialTrapped;
			trappingEventCounter += partialTrapped;
			//resetto le partialTrap
			partialTrapped = 0;
		}
		//stampa dei valori accumulati durante il 
		crDensPrint(crFile, crCountVsNorm, totalTrapped, tempo - 1);
		frDensPrint(frFile, frCountVsNorm, totalTrapped, tempo - 1);

		//reset di frame_dens/crown_dens dopo la stampa
		frCountVsNormReset(frCountVsNorm);
		crCountVsNormReset(crCountVsNorm);
		
	}
}
