/*
    Copyright (C) 2012  Leonardo Pozzati
    This file is part of MetastableRandWalk.

    MetastableRandWlak is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    MetastableRandWalk is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Nome-Programma.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MYLIBEVOLSMART_H
#define _MYLIBEVOLSMART_H

#include <list>
#include <map>
#include <utility>
#include <iostream>
#include "libPrint.h"
#include "libDens.h"


						//LIBEVOL//
//inizializzatore della configurazione delle particelle iniziali
//mette un numero arbitrario di particelle
//in posizioni random equiprobabili all'interno della griglia
std::list< std::pair <signed long, signed long> >
stateInit(unsigned long totalPart, unsigned long bound);

//inizializzatore della configurazione delle particelle iniziali
//mette un numero arbitrario di particelle
//in posizioni random equiprobabili all'interno della griglia
//saltando l'origine.
//l'origine viene popolata, in caso, in un secondo momento
std::list< std::pair <signed long, signed long> >
noTrappedStateInit(unsigned long totalPart, unsigned long bound);

//test sulle coordinate
int originCheck(std::pair <signed long, signed long> coord,
				std::pair <signed long, signed long> positionToCheck);

//elimina da stato le particelle nell'origine e le conta nel counter
void stateResize(std::list< std::pair <signed long, signed long> > &state,
				 unsigned long &trappedCounter);

int viscTest(long double num);

//emette una particella dalla trappola.
//la mette in coda allo stato
//e diminuisce il contatore delle particelle
void trapEmission(std::list< std::pair <signed long,
										signed long> > &state,
				  unsigned long &trappedCounter,
				  unsigned long &emissionCounter);

//muove le particelle contenute in stato
//rimuove le particelle che cadono nell'origine
//aggiorna il contatore
void makeSmartMove(signed long max,
				   unsigned long step,
				   std::list< std::pair <signed long,
										 signed long> > &state,
				   std::map<unsigned long,
							std::pair <unsigned long, unsigned long > >
				   &frCountVsNorm,
				   std::map<unsigned long,
							std::pair <unsigned long, unsigned long > >
				   &crCountVsNorm,
				   unsigned long &trappedPartCounter,
				   unsigned long &trappingEventCounter);

//evolve il sistema e stampa quando deve
void evolveAndPrint(std::ofstream &frFile,
					std::ofstream &crFile,
					std::list< std::pair <signed long,
										  signed long> > &state,
					unsigned long initialTrappedPart,
					unsigned long &emissionCounter,
					unsigned long &trappingEventCounter,
					unsigned long bound,
					unsigned long step,
					long double visc,
					unsigned long length1,
					unsigned long printTime1,
					unsigned seed);

#endif
