/*
    Copyright (C) 2012  Leonardo Pozzati
    This file is part of MetastableRandWalk.

    MetastableRandWlak is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    MetastableRandWalk is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Nome-Programma.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "libDens.h"
#include "libPrint.h"
#include "libEvolSmart.h"
#include "libSys.h"
#include "parseLib.h"
#include <sstream>


using namespace std;

int main(int argc, char *argv[]) {
	//parametri obbligatori
	string mandatory[] = {"-set", "-file", "-part", "-bound",
						  "-step", "-visc", "-length", "-print" 
						 };
	//parametri opzionali
	//valore obbligatorio:valore opzionale::senza valore
	string optional[] = {"-seed", ":", "-trap", "::"};
	int len1 = sizeof(mandatory) / sizeof(string);
	int len2 = sizeof(optional) / sizeof(string);
	
	//scan della riga di comando
	//e restituzione dei parametri, con valori,
	//nella mappa param
	map<string, string> param = inputScan(mandatory, len1, 
										  optional, len2,
										  argv, argc);

	//generazione del path della directory set
	string cartella;
	cartella = getcwd();
	int num;
	stringstream scartella(param["-set"]);
	scartella >> num;
	if(num <= 9) {
		cartella += "/set0";
	} else {
		cartella += "/set";
	}
	cartella += param["-set"];
	cartella += "-" + param["-bound"];
	cartella += "-" + param["-step"];
	cartella += "-" + param["-visc"];
	cartella += "-" + param["-part"];
	cartella += "-" + param["-length"];
	cartella += "-" + param["-print"];
	
	//creazione della directory set
	if(! exists(cartella)) {
		myMkdir(cartella);
	}

	//generazione del path della sottodirectory simulazione
	string file;
	stringstream sfile(param["-file"]);
	sfile >> num;
	if(num <= 9) {
		file = cartella + "/simulazione0";
	} else {
		file = cartella += "/simulazione";
	}
	file += param["-file"];
	
	//creazione della sottodirectory simulazione
	if(! exists(file)) {
		myMkdir(file);
	}

	//generazione del path dei file di output
	//e apertura in  modalità append
	string crFilePath = file + "/crown_dens.txt";
	ofstream crFile;
	crFile.open(crFilePath.c_str(), ios::app);

	string frFilePath = file + "/frame_dens.txt";
	ofstream frFile;
	frFile.open(frFilePath.c_str(), ios::app);

	string reportFilePath = file + "/events_count.txt";
	ofstream reportFile;
	reportFile.open(reportFilePath.c_str(), ios::app);

	//inizializzazione dei parametri iniziali per
	//il numero di particelle
	//il semilato della scacchiera
	unsigned long particles, bound;
	stringstream sparticles(param["-part"]);
	sparticles >> particles;
	stringstream sbound(param["-bound"]);
	sbound >> bound;
	//il passo per le corone e i frame
	//la durata della simulazione
	//l'intervallo di stampa
	//la viscosità della trappola
	unsigned long step, length, printTime;
	long double visc;
	stringstream sstep(param["-step"]); 
	stringstream slength(param["-length"]); 
	stringstream sprintTime(param["-print"]); 
	stringstream svisc(param["-visc"]); 
	sstep >> step;
	slength >> length;
	sprintTime >> printTime;
	svisc >> visc;

	//gestione del seme per le funzioni di generazione
	//di numeri casuali
	unsigned seed;
	if(param.find("-seed") != param.end()) {
		stringstream sseed(param["-seed"]);
		sseed >> seed;
	} else {
		seed = (unsigned) time(NULL);
	}
	srand48(seed);

	//inizializzazione
	//della lista di coordinate delle pedine
	//e dell'eventuale numero di particelle inizialmente intrappolate
	unsigned long initTrapped = 0;
	list< pair<signed long, signed long> > state;
	if(param.find("-trap") != param.end()) {
		//inizializzazione differenziata
		//per particelle nella griglia
		//e nella trappola
		stringstream strapped(param["-trap"]);
		if(param["-trap"] != "NaV")
			strapped >> initTrapped;
		state = noTrappedStateInit(particles, bound);		
	} else {
		//inizializzazione generica
		state = stateInit(particles, bound);
		stateResize(state, initTrapped);
	}
	
	//inizializzazione dei contatori per il file di report
	unsigned long emissionCounter = 0;
	unsigned long trappingEventCounter = 0;
	
	//pronti
	struct timeval start;
	//partenza
	gettimeofday(&start, NULL);
	//via
	evolveAndPrint(frFile, crFile,
				   state, initTrapped,
				   emissionCounter, trappingEventCounter,
				   bound, step, visc,
				   length, printTime, seed);

	//la simulazione è finita, quindi prendo il tempo
	struct timeval end;
	gettimeofday(&end, NULL);
	//calcolo la durata in millisecondi
	long mtime, seconds, useconds;
	seconds  = end.tv_sec  - start.tv_sec;
	useconds = end.tv_usec - start.tv_usec;
	mtime = ((seconds) * 1000 + useconds/1000.0);
	//scrivo il file di report
	reportPrint(reportFile, param, seed, mtime,
				"Intrappolamenti", trappingEventCounter,
				"Emissioni", emissionCounter);

	return 0;

}

